/*
Copyright (C) 2006, 2007 David Bateman
This file is part of Octave.
Octave is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either
version 3 of the License, or (at your option) any later
version.
Octave is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.
You should have received a copy of the GNU General Public
License along with Octave; see the file COPYING. If not,
see <http://www.gnu.org/licenses/>.*/

//Author: Mike Zielinski <mikez23@aol.com>



#include <octave/oct.h>
int testForMin(int index, RowVector x,Matrix means)
{
  dim_vector sizes=means.dims();
  if(index<0||index>=sizes(0))
    return 0;
  double testNorm=(x-means.row(index)).norm(2.0);
  
  for(int i=1;i<=3;i++)
    {
      if(index+i<sizes(0)-1)
	{ 
          if(testNorm>(x-means.row(index+i)).norm(2.0))
	    return 0;
	}
      if(index-i>0)
	{	
              if(testNorm>(x-means.row(index-i)).norm(2.0))
                return 0;
        }
    }
  return 1;
}
int getClosestColor(RowVector x,Matrix means)
{
   
  


 dim_vector sizes=means.dims();
 
 double xToBottom=((x.norm(2.0))-(means.row(0).norm(2.0)));
 double topToBottom=((means.row(sizes(0)-1).norm(2.0))-(means.row(0).norm(2.0)));
 if(xToBottom<0)
   xToBottom=0;
 assert(topToBottom>0); 
double lambda= xToBottom/topToBottom;
 int index=(int)(lambda*sizes(0));
 if(index>sizes(0)-1)
   index=sizes(0)-1;
 
 for(int i=0;i<sizes(0);i++)
   {
     if(index!=0)
       {
      if( testForMin(index-i,x,means)==1)
	//	octave_stdout<<index-i<<'\n';
         return index-i;
       }
     if(index!=sizes(0)-1)
       {
     if(testForMin(index+i,x,means)==1)
       {
	 // octave_stdout<<index+i<<'\n';
	  return index+i;

       }
       }    
   } 
 return 0;//to get rid of warning
}

Matrix getClosestColors(Matrix Red, Matrix Green, Matrix Blue,Matrix colors)
{
  dim_vector imageDims= Red.dims();
  
  
  Matrix image(imageDims);
  for(int i=0;i<imageDims(0);i++)
    {  for(int j=0;j<imageDims(1);j++)
      {
        RowVector pixel(3);
        pixel(0)=Red(i,j);
        pixel(1)=Green(i,j);
        pixel(2)=Blue(i,j);
        image(i,j)=getClosestColor(pixel,colors);
      }
    }
  return image;
}
  

DEFUN_DLD (getclosestcolors, args, nargout ,
"-*- texinfo -*-\n\
@deftypefn {Loadable Function} {@var{I} =} getclosestcolors(@var{R},@var(G),@var(B),@var(colormap))\n\
\n\
used in rgb2ind2 \n\
\n\
The image is returned as a matrix of dimension MxN  matrix\n\
for  RGB and gray-level  images for a given colormap\n\
\n\
@end deftypefn\n\
@seealso{rgb2ind2}")
{ 
  octave_value_list ret_value;
  int nargin  = args.length();
  if(nargin!=4)
    {
      print_usage();
      return ret_value;
    }
  
  Matrix Red  =args(0).matrix_value();
  Matrix Green = args(1).matrix_value();
  Matrix Blue = args(2).matrix_value();
  Matrix colors=args(3).matrix_value();
  if(!error_state)
    {
       Matrix image=getClosestColors(Red,Green,Blue,colors);
       return octave_value(image);
    }
  return ret_value;
}
