## Copyright (C) 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2004, 2005,
##               2006, 2007 John W. Eaton
##
## This file is part of Octave.
##
## Octave is free software; you can redistribute it and/or modify it
## under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 3 of the License, or (at
## your option) any later version.
##
## Octave is distributed in the hope that it will be useful, but
## WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
## General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with Octave; see the file COPYING.  If not, see
## <http://www.gnu.org/licenses/>.

## -*- texinfo -*-
## @deftypefn  {Function File} {[@var{x}, @var{map}] =} rgb2ind2 (@var{rgb})
## @deftypefnx {Function File} {[@var{x}, @var{map}] =} rgb2ind2 (@var{r}, @var{g}, @var{b})
## Convert an RGB image to an Octave indexed image.
## @seealso{ind2rgb, rgb2ntsc}
## @end deftypefn


## Author: Tony Richardson <arichard@stark.cc.oh.us>
## Created: July 1994
## Adapted-By: jwe
## Adapted-By: Mike Zielinski <mikez23@aol.com>

function [X, map] = rgb2ind2 (R, G, B)

  if (nargin != 1 && nargin != 3)
    print_usage ();
  endif
  
  if (nargin == 1)
    rgb = R;
    if (length (size (rgb)) == 3 && size (rgb, 3) == 3)
      R = rgb(:,:,1);
      G = rgb(:,:,2);
      B = rgb(:,:,3);
    else
      error ("rgb2ind: argument is not an RGB image");
    endif
  endif

  if (! size_equal (R, G) || ! size_equal (R, B))
    error ("rgb2ind: arguments must all have the same size");
  endif

  [hi, wi] = size (R);

  X = zeros (hi, wi);
  
  mapholder = zeros (hi*wi, 3);
typestring=typeinfo(R);
divider =1;
##for arrays read in  as jpeg and png we have
if(strcmp(typestring,"uint8 matrix")==1)
  divider=255;
elseif(strcmp(typestring,"uint16 matrix")==1)
  divider=65535;
endif
mapholder(:,1) = cast(R(:),"double")/divider;
mapholder(:,2) = cast( G(:),"double")/divider;
mapholder(:,3) = cast(B(:),"double")/divider;
if(size(mapholder,1)>1024)
  mapholder=unique(mapholder,"rows");
  averageSize=uint32(floor((size(mapholder,1)/1024)));
  for i=1:1024
	holder =zeros(averageSize,3); 
    for j=1:averageSize
	    holder(j,:)=double(mapholder((i-1)*averageSize+j,:));
    endfor
    maphlder(i,:)=  sum(holder,1)/double(averageSize);
endfor
endif
maphlder=sortByNorm(maphlder);
##get closest color needs a sorted color map
X=getclosestcolors(double(R)/divider,double(G)/divider,double(B)/divider,maphlder);      
map=maphlder;
endfunction
